/*
 * RFM12B.c
 *
 *  Created on: 10-06-2013
 *      Author: Piotr Rzeszut
 *  
 * Description: Funkcje do obsugi ukadu RFM12B z poziomu procesora AVR
 */
#include "RFM12B.h"

void Rfm_spi_init(void){
	DDRB |= (1<<PB7)|(1<<PB5)|(1<<PB4);//konfiguracja kierunku i podcigania linii SPI
	PORTB |= (1<<PB6);
	//uwaga - pomimo i tu nie uywamy lini SS jako lini CS dla ukadu RFM
	//to do poprawnej pracy moduu SPI w procesorze linia SS musi by ustawiona jako wyjcie

	SPCR |= (1<<SPE)|(1<<MSTR)|(1<<SPR0);//SPI w trybie master z podziaem zegara przez 64 
	

	PORTC |= (1<<PC5);//dezaktywacja ukadu RFM12B
	DDRC |= (1<<PC5);//konfiguracja kierunku linii CS
}

uint16_t Rfm_xmit(uint16_t data){
	//na pocztek rodzielamy dane na 2 bajty
	uint8_t msb, lsb;
	lsb = data;
	msb = data>>8;
	PORTC &= ~(1<<PC5);//aktywujemy lini CS ukadu
	//teraz wysyamy 2 bajty jednoczenie odbierajc 2 bajty z ukadu
	SPDR = msb;
	while(!(SPSR&(1<<7)));
	msb = SPDR;
	SPDR = lsb;
	while(!(SPSR&(1<<7)));
	lsb = SPDR;
	//_delay_ms(1);
	PORTC |= (1<<PC5);//dezaktywujemy lini CS ukadu
	//i zwracamy odebrane dane
	return( (((uint16_t)msb)<<8)+((uint16_t)lsb) );
}

void Rfm_ready_wait(void){
	//aby sprawdzi gotowo ukadu do odbioru komend z procesora
	//aktywujemy jego lini CS
	PORTC &= ~(1<<PC5);
	//czekamy a ukad na to zareaguje
	_delay_us(1);
	//i oczekujemy a linia danych zostanie zwolniona, co oznacza wykonanie poprzednich polece
	/*
	w wersji docelowej ta instrukcja powinna by wykonywana do okrelonego czasu, po ktrym
	procesor powinien uzna, e ukad si zawiesi lub nie odpowiada z innych przyczyn
	i naley podj odpowiednie dziaania (np. informacja, e adne dane nie sa do nas wysyane lub
	e nalezy zreseowa ukad)
	*/
	while(!(PINB&(1<<PB6)));
}

void Rfm_write_str(char* s, uint8_t address){
	//instrukcja nadajca cig znakw
	char c;//zmienna pomocnicza
	//wczamy nadajnik
	Rfm_xmit(POWER|EN_TRANSMISSION|EN_SYNTH|EN_OSC|DIS_CLKO);
	//czekamy nieco duzej na jego aktywacj
	_delay_ms(5);

	//nadajemy 3 bajty synchronizacji (0xAA)
	Rfm_ready_wait();
	Rfm_xmit(TX_WRITE|0xAA);
	Rfm_ready_wait();
	Rfm_xmit(TX_WRITE|0xAA);
	Rfm_ready_wait();
	Rfm_xmit(TX_WRITE|0xAA);
	//nastpnie kolejny bajt synchrnoizacji (0x2D)
	Rfm_ready_wait();
	Rfm_xmit(TX_WRITE|0x2D);
	Rfm_ready_wait();
	//oraz definiowalny bajt synchronizacji (czyli nasz adres)
	Rfm_xmit(TX_WRITE|address);
	Rfm_ready_wait();

	//wysyamy cay acuch znakw
	while((c=*s++)){
		Rfm_xmit(TX_WRITE|c);
		Rfm_ready_wait();
	}
	//i bajt rwny 0 koczcy cig znakw
	Rfm_xmit(TX_WRITE|0);
	Rfm_ready_wait();
	//oraz 3 bajty koczce transmisj
	Rfm_xmit(TX_WRITE|0xAA);
	Rfm_ready_wait();
	Rfm_xmit(TX_WRITE|0xAA);
	Rfm_ready_wait();
	Rfm_xmit(TX_WRITE|0xAA);
	Rfm_ready_wait();
	//nastpnie wyczamy nadajnik
	Rfm_xmit(POWER|DIS_CLKO);
}

void Rfm_read_str(char* c, uint8_t max_size){
	//wczamy odbiornik
	Rfm_xmit(POWER|EN_RX|EN_BASEBAND|EN_OSC|DIS_CLKO);
	_delay_ms(5);
	//para komend powodujca w efekcie dziaania reset synchronizacji odbiornika
	Rfm_xmit(FIFO_RST|FIFO_IT_8|HS_RST_DIS);
	Rfm_xmit(FIFO_RST|FIFO_IT_8|EN_AFT_SYNC|HS_RST_DIS);
	
	uint8_t i=0;
	do{
		//oczekujemy a ukad bdzie gotowy (a bajt bdzie w buforze)
		Rfm_ready_wait();
		//i odczytujemy ten bajt
		*c=(char)Rfm_xmit(FIFO_READ);
		i++;
		if(i>=max_size)*(++c)=0;
	}while(*c++!=0);

	//nastpnie wyczamy odbiornik
	Rfm_xmit(POWER|DIS_CLKO);
}
