/*----------------------------------------------------------------------*/
/* FAT file system sample project for FatFs            (C)ChaN, 2013    */
/*----------------------------------------------------------------------*/


#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <string.h>
#include "uart.h"
#include "xitoa.h"
#include "ff.h"
#include "diskio.h"
#include "rtc.h"
//#include "sound.h"


//FUSES = {0xAF, 0xC3, 0xFF};		
/* ATmega64 fuses: Low, High, Extended.
This is the fuse settings for this project. The fuse bits will be included
in the output hex file with program code. However some old flash programmers
cannot load the fuse bits from hex file. If it is the case, remove this line
and use these values to program the fuse bits. */



DWORD AccSize;				/* Work register for fs command */
WORD AccFiles, AccDirs;
FILINFO Finfo;

//char Line[80];				/* Console input buffer */
BYTE Buff[256];			/* Working buffer */

FATFS Fatfs;		/* File system object for each logical drive */
FIL File;				/* File object */
DIR Dir;					/* Directory object */


//BYTE RtcOk;					/* RTC is available */

volatile UINT Timer;		/* Performance timer (100Hz increment) */



/*---------------------------------------------------------*/
/* 100Hz timer interrupt generated by OC2                  */
/*---------------------------------------------------------*/


ISR(TIMER2_COMP_vect)
{
	Timer++;			/* Performance counter for this module */
	disk_timerproc();	/* Drive timer procedure of low level disk I/O module */
}



/*---------------------------------------------------------*/
/* User Provided Timer Function for FatFs module           */
/*---------------------------------------------------------*/
/* This is a real time clock service to be called from     */
/* FatFs module. Any valid time must be returned even if   */
/* the system does not support a real time clock.          */
/* This is not required in read-only configuration.        */


DWORD get_fattime (void)
{
	RTC rtc;

	rtc.year = 1994;
	rtc.month = 10;
	rtc.mday = 25;
	rtc.hour = 8;
	rtc.min = 1;
	rtc.sec = 54;

	/* Pack date and time into a DWORD variable */
	return	  ((DWORD)(rtc.year - 1980) << 25)
			| ((DWORD)rtc.month << 21)
			| ((DWORD)rtc.mday << 16)
			| ((DWORD)rtc.hour << 11)
			| ((DWORD)rtc.min << 5)
			| ((DWORD)rtc.sec >> 1);
}


static
void ioinit (void)
{
	/* Pull-up all GPIO pins */
	PORTA = 0b11111111;
	PORTB = 0b11111111;
	PORTC = 0b11111111;
	PORTD = 0b11111111;

	/* Start 100Hz system timer (TC2.OC) */
	OCR2 = F_CPU / 1024 / 100 - 1;
	TCCR2 = 0b00001101;
	TIMSK |= _BV(OCIE2);

	sei();
}

static
void get_line (char *buff, int len)
{
	BYTE c;
	int i = 0;


	for (;;) {
		c = uart_getc();
		if (c == '\r') break;
		if ((c == '\b') && i) {
			i--;
			uart_putc(c);
			continue;
		}
		if (c >= ' ' && i < len - 1) {	/* Visible chars */
			buff[i++] = c;
			xputc(c);
		}
	}
	buff[i] = 0;
	uart_putc('\n');
}


/*-----------------------------------------------------------------------*/
/* Main                                                                  */


int main (void)
{
	//char *ptr, *ptr2;
	//long p1, p2, p3;
	//BYTE res, b1, *bp;
	//UINT s1, s2, cnt;
	//DWORD ofs, sect = 0, blk[2];
	//FATFS *fs;
	//RTC rtc;


	ioinit();				/* Initialize port settings and start system timer process */
	uart_init(9600);		/* Initialize UART driver */
	xdev_out(uart_putc);	/* Register uart_putc() to xitoa module as console output */
	
	xputs(PSTR("\r\n\r\nSD read/write demo for And-Tech EvB boards\r\n"));


	if(disk_initialize(0)!=0){//inicjalizujemy warstw sprztow
		xputs(PSTR("SD card ERROR\r\n"));
		while(1);
	}


	xputs(PSTR("SD card initialized\r\n"));

	if(f_mount(0,&Fatfs)!=FR_OK){//inicjalizujemy system plikw
		xputs(PSTR("File System ERROR\r\n"));
		while(1);
	}

	xputs(PSTR("File system FATfs initialized\r\n"));

	if(f_open(&File,"SD.txt",FA_READ|FA_OPEN_ALWAYS)!=FR_OK){//otwieramy plik do odczytu, 
		//jeli plik nie istnieje to tworzymy nowy plik
		xputs(PSTR("File Open for reading ERROR\r\n"));
		while(1);
	}

	xputs(PSTR("File SD.txt opened for reading\r\n"));

	UINT i=0,k=0;	

	if(f_read(&File, &Buff, 255, &i)==FR_OK){//odczytujemy maksymalnie 255 znakw z pliku
		xprintf(PSTR("%u bytes read: "), i);//i je wywietlamy	
		uint8_t j=0;
		while(Buff[j]){
			uart_putc(Buff[j++]);
		}
		xputs(PSTR("\r\n"));
	}else{
		xputs(PSTR("File Read ERROR\r\n"));
	}

	if(f_close(&File)!=FR_OK){//zamykamy plik
		xputs(PSTR("File Close ERROR\r\n"));
		while(1);
	}

	xputs(PSTR("File Closed\r\n"));

	xputs(PSTR("Save something? (Y/N): "));//pytamy o to czy uytkownik chce co zapisa

	get_line((char*)Buff,255);
	xputs(PSTR("\r\n"));

	if(Buff[0]!='Y'){
		xputs(PSTR("End program\r\n"));
		while(1);
	}

	xputs(PSTR("Clear file? (Y/N): "));//oraz czy chce wyczyci plik

	get_line((char*)Buff,255);
	xputs(PSTR("\r\n"));

	if(Buff[0]=='Y'){
		if(f_open(&File,"SD.txt",FA_WRITE|FA_CREATE_ALWAYS)!=FR_OK){//jeli chce wyczyci plik to to czynimy
			xputs(PSTR("File Open for writing ERROR\r\n"));
			while(1);
		}	
	}else{
		if(f_open(&File,"SD.txt",FA_WRITE|FA_OPEN_ALWAYS)!=FR_OK){//a jeli chce co dopisa na kocu to otwieramy plik w odpowiednim trybie
			xputs(PSTR("File Open for writing ERROR\r\n"));
			while(1);
		}
		if(f_lseek(&File, f_size(&File))!=FR_OK){//i potem ustawiamy znacznik zapisu na kocu pliku
			xputs(PSTR("Cannot append data\r\n"));
			if(f_close(&File)!=FR_OK){
				xputs(PSTR("File Close ERROR\r\n"));
				while(1);
			}
			while(1);
		}
	}

	xputs(PSTR("File Opened for writing\r\n"));


	xputs(PSTR("Type text to write: "));

	get_line((char*)Buff,255);//pobieramy tekst od uytkownika
	xputs(PSTR("\r\n"));
	
	i=0;
	while(Buff[i])i++;

	if(f_write(&File,Buff,i,&k)!=FR_OK){//i zapisujemy go do pliku
		xputs(PSTR("File Write ERROR\r\n"));
	}

	xprintf(PSTR("%u bytes written\r\n"), i);

	if(f_close(&File)!=FR_OK){//po czym plik zamykamy
		xputs(PSTR("File Close ERROR\r\n"));
		while(1);
	}

	xputs(PSTR("File Closed\r\n"));

	xputs(PSTR("End program\r\n"));//teraz mona wyj kart i pobawi si na komputerze


	while(1);

}

